from sqlalchemy import create_engine, Column, Integer, String, DateTime, Table, \
    MetaData, or_, func, and_
from sqlalchemy.ext.declarative import declarative_base
from sqlalchemy.orm import sessionmaker, session, scoped_session

# 建立与MySQL的连接
engine = create_engine('mysql+pymysql://root:123456@localhost/woniunote', echo=False)

# 定义模型类继承的父类及数据连接会话
DBsession = sessionmaker(bind=engine)
dbsession = scoped_session(DBsession)   # 线程安全
Base = declarative_base()
md = MetaData(bind=engine)

# 定义模型类
# class Users(Base):
#     __tablename__ = "userx"
#
#     # 如果需要在SQLAlchemy里面直接创建表结构，则详细定义列
#     userid = Column(Integer, primary_key=True)
#     username = Column(String(50))
#     password = Column(String(32))
#     nickname = Column(String(30))
#     qq = Column(String(15))
#     role = Column(String(10))
#     credit = Column(Integer)
#     createtime = Column(DateTime)
#     updatetime = Column(DateTime)
#
# Users.metadata.create_all(engine)   # 创建表


class Users(Base):
    __table__ = Table('users', md, autoload=True)

class Article(Base):
    __table__ = Table('article', md, autoload=True)

class Comment(Base):
    __table__ = Table('comment', md, autoload=True)


if __name__ == '__main__':
    # 通过query和filter进行标准查询, all返回多行数据的列表
    # result = dbsession.query(Users).filter(Users.userid>=5).all()
    # result = dbsession.query(Users).filter_by(userid=5).all()
    # print(result)
    # print(result[0].username)
    # for row in result:
    #     print(row.userid, row.username)

    # first()只返回一条数据，直接返回模型类
    # row = dbsession.query(Users).filter(Users.userid <5).first()  # ==> limit 0,1
    # print(row)
    # print(row.userid, row.username)

    # 直接指定列名，则返回的是元组，不再是模型对象
    # row = dbsession.query(Users.userid, Users.username).filter(Users.userid < 5).first()
    # print(row)
    # print(row.userid, row.username)

    # 新增
    # user = Users(username='reader5@woniuxy.com',
    #              password='e10adc3949ba59abbe56e057f20f883e', role='user', credit=5)
    # dbsession.add(user)
    # dbsession.commit()      # 修改类操作需要手工提交

    # 删除和更新：需要查询出来要修改的行，再进行删除或修改
    # row = dbsession.query(Users).filter_by(userid=9).first()
    # row.username = 'reader6@woniuxy.com'
    # dbsession.commit()

    # row = dbsession.query(Users).filter_by(userid=9).delete()
    # dbsession.commit()


    # 基础查询汇总, 直接打印一个类的时候，具体打印什么内容，由类的__repr__魔术方法决定，可以重写
    # select * from users
    result = dbsession.query(Users).all()
    # select userid, username from users
    result = dbsession.query(Users.userid, Users.username).all()
    # select * from users where userid=1 and qq='12345678'
    result = dbsession.query(Users).filter_by(userid=1, qq='123456789').all()
    # select * from users where userid>7 or nickname='丹尼'
    result = dbsession.query(Users).filter(or_(Users.userid>7, Users.nickname=='丹尼')).all()
    # select * from users limit 3
    result = dbsession.query(Users).limit(3).all()
    # select * from users limit 3,5
    result = dbsession.query(Users).limit(5).offset(3).all()
    # select count(*) from users where ....
    count = dbsession.query(Users).filter(Users.userid > 3).count()
    print(count)
    # select distinct(qq) from users
    result = dbsession.query(Users.qq).distinct(Users.qq).all()
    # select * from users order by userid desc
    result = dbsession.query(Users).order_by(Users.userid.desc()).all()
    # select * from users where username like '%qiang%'
    result = dbsession.query(Users).filter(Users.username.like('%qiang%')).all()
    # select * from users group by role
    result = dbsession.query(Users).group_by(Users.role).all()
    result = dbsession.query(Users).group_by(Users.role).having(Users.userid>2).all()
    # 聚合函数：min, max, avg, sum
    # select sum(credit) from users
    result = dbsession.query(func.sum(Users.credit)).first()
    # filter: == >= > <= < != in not

    # for row in result:
    #     print(row.userid, row.username)

    # 多表连接查询: select * from article inner join users on article.userid=users.userid where article.articleid=1
    # 多表连接查询时，返回的结果集不再是单纯的[Model, Model]数据结构，而是每张表的结果有独立的对象来维护
    # result = dbsession.query(Article, Users).join(Users, Article.userid == Users.userid).filter(Article.articleid==1).all()
    # result = dbsession.query(Article.articleid, Article.headline, Users.nickname).join(Users, Article.userid == Users.userid).filter(Article.articleid==1).all()
    #
    # print(result)
    # for articleid, headline, nickname in result:
    #     print(articleid, headline, nickname)

    # 外连接：查询每一个用户发表过的文章的阅读总量,　outerjoin默认为左外连接
    # select users.userid, users.nickname, sum(article.readcount) as total from users left join article
    # on users.userid=article.userid group by (users.userid)
    result = dbsession.query(Users.userid, Users.nickname, func.sum(Article.readcount))\
        .outerjoin(Article, Users.userid==Article.userid).group_by(Users.userid).all()
    print(result)

    # 复杂查询: and和or混用，username like 'qiang' or (userid>3 and nickname='reader3')
    result = dbsession.query(Users).filter(or_(Users.username.like('%qiang%'), and_(Users.userid>3, Users.nickname=='reader3'))).all()
    result = dbsession.query(Users).filter(and_(Users.username.like('%qiang%'), or_(Users.userid>3, Users.nickname=='reader3'))).all()
    result = dbsession.query(Users).filter(Users.username.like('%qiang%'), or_(Users.userid>3, Users.nickname=='reader3')).all()
    for row in result:
        print(row.userid, row.username)

    # 三表连接
    result = dbsession.query(Comment, Users).join(Users, Comment.userid==Users.userid)\
        .join(Article, Article.articleid==Comment.articleid).all()
    print(result)

    # 利用SQLAlchemy执行原生SQL
    result = dbsession.execute("select * from users where userid>5").fetchall()
    print(result)
    print(result[0].username)

    dbsession.execute("delete from users where userid=10")
    dbsession.commit()


    result = dbsession.query(Users).all()
    print(result)
    list = []
    for row in result:
        dict = {}
        for k,v in row.__dict__.items():
            if not k.startswith('_sa_instance_state'):
                dict[k] = v
        list.append(dict)
    print(list)