import pymysql
from pymysql.cursors import DictCursor

class MySQL:
    # 实例化即创建与数据库之间的连接
    def __init__(self):
        conn = pymysql.connect(host='127.0.0.1', port=3306, user='root', password='123456',
                       charset='utf8', database='woniunote', autocommit=True)
        self.cursor = conn.cursor(DictCursor)

    # 封装基础查询语句
    def query(self, sql):
        self.cursor.execute(sql)
        result = self.cursor.fetchall()
        return result

    # 执行修改操作
    def execute(self, sql):
        try:
            self.cursor.execute(sql)
            return 'OK'
        except:
            return 'Fail'

# 封装成标准的模型类，供子类继承
# 增加field()方法来指定查询哪些列，*代表所有列
class Model:
    def __init__(self, **kwargs):
        for k, v in kwargs.items():
            self.__setattr__(k, v)

    # 通过链式操作指定查询哪些列
    def field(self, columns):
        self.columns = columns  # 动态增加类实例属性
        return self

    # 带列名的查询条件
    def select(self, **where):
        table = self.__class__.__getattribute__(self, 'table_name')

        if hasattr(self, 'columns'):
            sql = "select %s from %s" % (self.columns, table)
        else:
            sql = "select * from %s" % table

        if where is not None:
            sql += " where"
            for k, v in where.items():
                sql += " %s='%s' and" % (k, v)
            sql += ' 1=1'

        result = MySQL().query(sql)
        return result

    # 正常新增数据
    def insert(self):
        keys = []
        values = []
        for k, v in self.__dict__.items():
            keys.append(k)
            values.append(str(v))

        sql = "insert into %s(%s) values('%s')" % (self.table_name, ','.join(keys), "','".join(values))
        result = MySQL().execute(sql)
        print(result)


# 定义子类Users和Article模型类
class Users(Model):
    table_name = 'users'

    # 调用父类的构造方法
    def __init__(self, **kwargs):
        super().__init__(**kwargs)


class Article(Model):
    table_name = 'article'

    # 调用父类的构造方法
    def __init__(self, **kwargs):
        super().__init__(**kwargs)

user = Users()
# result = user.select(userid=1)
result = user.field('userid, username, nickname').select(userid=2)
print(result)

article = Article()
result = article.select(articleid=1)
print(result[0]['headline'])