from flask import Flask, render_template, request, redirect, url_for, session, make_response
import os

app = Flask(__name__, template_folder='template', static_url_path='/', static_folder='resource')
app.config['SECRET_KEY'] = os.urandom(24)   #生成随机数种子，用于产生SessionID

# 定义首页地址
@app.route('/')
def index():
    return render_template('index.html')

# 定义文章阅读页面地址
# @app.route('/article')
# def read():
#     return render_template('article-user.html')

# 定义用户注册接口
@app.route('/user/reg', methods=['POST', 'PUT'])
def register():
    return 'Good'

# 请求参数的问题：
# URL地址自带的查询参数：http://127.0.0.1:5000/test?username=woniuxy&password=123456
# 通过Flask自定义的路由规则和参数：http://www.woniunote.com/article/<articleid>
# POST请求的参数，通过Post正文传输。

# 查询参数：
@app.route('/test')
def test():
    username = request.args.get('username')
    password = request.args.get('password')
    return '你的用户名为：%s，密码为：%s' % (username, password)

# 路由地址参数，路由地址参数必须同步定义在接口函数里面作为该参数的形参
# 由于articleid是一个整数，所以可以在定义路由参数时，直接用int进行类型限制
@app.route('/article/<int:articleid>-<page>')
def article(articleid, page):
    return f"你正在访问编号为：{articleid} 的文章，目前是第 {page} 页."

# 直接读取Post请求正文参数
@app.route('/user/login', methods=['POST'])
def login():
    username = request.form.get('username')
    password = request.form.get('password')
    if username == 'woniuxy' and password == '123456':
        return '登录成功'
    else:
        return '登录失败'

# 定义满足RESTful风格的接口地址
@app.route('/article', methods=['GET'])
def view():
    return '获取文章'

@app.route('/article', methods=['POST'])
def add():
    return '新增文章'

@app.route('/article/123', methods=['PUT'])
def update():
    # 同样可以带请求正文
    return '修改文章'

@app.route('/article/123', methods=['GET'])
def get():
    return '读取文章123'

@app.route('/article/123', methods=['DELETE'])
def delete():
    return '删除文章'

# 如何利用RESTful风格来定义用户登录和注册接口
# 用户注册：POST->/user
# 用户登录：POST->/session
# 用户注销：DELETE->/session

# POST->/article/add

# URL重定向的问题：是HTTP协议本身的功能，重定向302状态码，在响应头里面通过指定Location字段来告诉浏览器，跳转地址
@app.route('/red')
def red():
    # return redirect(url_for('index')) # 使用项目中的函数名进行重定向，通过url_for构建重定向地址
    return redirect('/')        # 直接指定重定向地址

# 直接在响应中进行重定向
@app.route('/redjs')
def redjs():
    html = '感谢访问，2秒以后将跳转到首页.'
    html += "<script>"
    html += "setTimeout(function() {location.href='/';}, 2000);"
    html += "</script>"
    return html

# Session和Cookie
# 要处理Session，则必须为app实例设置SECRET_KEY配置参数，配置随机数生成器（Session ID），再使用session函数进行操作
# 要处理Cookie，需要使用response对象来往HTTP的响应中写入满足HTTP协议的Cookie要求的信息（Key，Value，Age）
@app.route('/session')
def sess():
    session['islogin'] = 'true'
    session['username'] = 'woniuxy'
    session['nickname'] = '强哥'
    session['role'] = 'editor'

    return '当前Session的用户名为：%s' % session.get('username')

# 利用自定义响应的方式来往浏览器设置Cookie
@app.route('/cookie')
def cookie():
    response = make_response("这是设置Cookie的操作")
    response.set_cookie('username', 'qiang', max_age=30)
    response.set_cookie('password', '123456', max_age=30)

    # request.cookies.get('username') 无法在同一个接口中既设置cookie又获取cookie
    return response

@app.route('/sc/read')
def scread():
    # return '你当前的昵称为：%s' % session.get('username')
    return '你当前的昵称为：%s' % request.cookies.get('username')

# 引用demo.py
from controller.demo import *
app.register_blueprint(demo)    # 使用Blueprint时，必须将其注册到app中


# 针对app实例定义全局拦截器
# @app.before_request
def before():
    # 如果用户已经登录session['islogin']='true'，则不作拦截
    url = request.path  # 读取到当前接口的地址
    if url == '/session':
        pass
    elif session.get('islogin') != 'true':
        return '你当前还没有登录，无法访问该接口'
    else:
        pass

    # 白名单：所有静态资源（JS，Image，CSS等）必须设置为白名单
    # 登录或者跟用户操作有权限要求的接口前要做的操作，必须放行
    # 在列表中设置白名单（当然，如果业务需要，也可以设置黑名单
    pass_list = ['/', '/reg', '/login', '/vcode', '/session']
    # 静态资源如图片，CSS和JS代码等，可以通过后缀名来进行放行
    suffix = url.endswith('.png') or url.endswith('.jpg') or \
             url.endswith('.css') or url.endswith('.js')
    if url in pass_list or suffix:
        pass  # 如果在白名单中，则旅行请求，不做拦截
    else:
        # 开始实现拦截逻辑
        pass


if __name__ == '__main__':
    # from controller.demo import *
    # app.register_blueprint(demo)  # 使用Blueprint时，必须将其注册到app中
    app.run(debug=True)
